#ifndef _COMPAT_XSM_FLASK_OP_H
#define _COMPAT_XSM_FLASK_OP_H
#include <xen/compat.h>
#include <public/xsm/flask_op.h>
#pragma pack(4)
#include "../event_channel.h"
#pragma pack(4)

struct compat_flask_load {
    COMPAT_HANDLE(char) buffer;
    uint32_t size;
};

struct compat_flask_setenforce {
    uint32_t enforcing;
};

struct compat_flask_sid_context {

    uint32_t sid;

    uint32_t size;
    COMPAT_HANDLE(char) context;
};

struct compat_flask_access {

    uint32_t ssid;
    uint32_t tsid;
    uint32_t tclass;
    uint32_t req;

    uint32_t allowed;
    uint32_t audit_allow;
    uint32_t audit_deny;
    uint32_t seqno;
};

struct compat_flask_transition {

    uint32_t ssid;
    uint32_t tsid;
    uint32_t tclass;

    uint32_t newsid;
};

struct compat_flask_userlist {

    uint32_t start_sid;

    uint32_t size;
    union {

        COMPAT_HANDLE(char) user;

        COMPAT_HANDLE(uint32) sids;
    } u;
};

struct compat_flask_boolean {

    uint32_t bool_id;

    uint8_t enforcing;

    uint8_t pending;

    uint8_t new_value;

    uint8_t commit;

    uint32_t size;

    COMPAT_HANDLE(char) name;
};

struct compat_flask_setavc_threshold {

    uint32_t threshold;
};

struct compat_flask_hash_stats {

    uint32_t entries;
    uint32_t buckets_used;
    uint32_t buckets_total;
    uint32_t max_chain_len;
};

struct compat_flask_cache_stats {

    uint32_t cpu;

    uint32_t lookups;
    uint32_t hits;
    uint32_t misses;
    uint32_t allocations;
    uint32_t reclaims;
    uint32_t frees;
};

struct compat_flask_ocontext {

    uint32_t ocon;
    uint32_t sid;
    uint64_t low, high;
};

struct compat_flask_peersid {

    evtchn_port_compat_t evtchn;

    uint32_t sid;
};

struct compat_flask_relabel {

    uint32_t domid;
    uint32_t sid;
};

struct compat_flask_devicetree_label {

    uint32_t sid;
    uint32_t length;
    COMPAT_HANDLE(char) path;
};

struct compat_flask_op {
    uint32_t cmd;
    uint32_t interface_version;
    union {
        struct compat_flask_load load;
        struct xen_flask_setenforce enforce;

        struct compat_flask_sid_context sid_context;
        struct xen_flask_access access;

        struct xen_flask_transition transition;
        struct compat_flask_userlist userlist;

        struct compat_flask_boolean boolean;
        struct xen_flask_setavc_threshold setavc_threshold;
        struct xen_flask_hash_stats hash_stats;
        struct xen_flask_cache_stats cache_stats;

        struct xen_flask_ocontext ocontext;
        struct xen_flask_peersid peersid;
        struct xen_flask_relabel relabel;
        struct compat_flask_devicetree_label devicetree_label;
    } u;
};
typedef struct compat_flask_op compat_flask_op_t;
DEFINE_COMPAT_HANDLE(compat_flask_op_t);
#pragma pack()
#endif /* _COMPAT_XSM_FLASK_OP_H */
